@@ -1,6 +1,3 @@
-#ifndef lint
-static const char SCCSID[]="@(#)PJ_wag2.c	4.1	94/02/15	GIE	REL";
-#endif
 #define PJ_LIB__
 # include	<projects.h>
 PROJ_HEAD(wag2, "Wagner II") "\n\tPCyl., Sph.";
@@ -9,7 +6,7 @@ PROJ_HEAD(wag2, "Wagner II") "\n\tPCyl., Sph.";
 #define C_p1 0.88022
 #define C_p2 0.88550
 FORWARD(s_forward); /* spheroid */
-	lp.phi = aasin(C_p1 * sin(C_p2 * lp.phi));
+	lp.phi = aasin(P->ctx,C_p1 * sin(C_p2 * lp.phi));
 	xy.x = C_x * lp.lam * cos(lp.phi);
 	xy.y = C_y * lp.phi;
 	return (xy);
@@ -17,7 +14,7 @@ FORWARD(s_forward); /* spheroid */
 INVERSE(s_inverse); /* spheroid */
 	lp.phi = xy.y / C_y;
 	lp.lam = xy.x / (C_x * cos(lp.phi));
-	lp.phi = aasin(sin(lp.phi) / C_p1) / C_p2;
+	lp.phi = aasin(P->ctx,sin(lp.phi) / C_p1) / C_p2;
 	return (lp);
 }
 FREEUP; if (P) pj_dalloc(P); }
