@@ -1,7 +1,4 @@
 /* Convert DMS string to radians */
-#ifndef lint
-static const char SCCSID[]="@(#)dmstor.c	4.4	93/06/16	GIE	REL";
-#endif
 #include <projects.h>
 #include <string.h>
 #include <ctype.h>
@@ -20,6 +17,11 @@ vm[] = {
 };
 	double
 dmstor(const char *is, char **rs) {
+	return dmstor_ctx( pj_get_default_ctx(), is, rs );
+}
+
+	double
+dmstor_ctx(projCtx ctx, const char *is, char **rs) {
 	int sign, n, nl;
 	char *p, *s, work[MAX_WORK];
 	double v, tv;
@@ -49,7 +51,7 @@ dmstor(const char *is, char **rs) {
 			n = 2; break;
 		case 'r': case 'R':
 			if (nl) {
-				pj_errno = -16;
+				pj_ctx_set_errno( ctx, -16 );
 				return HUGE_VAL;
 			}
 			++s;
@@ -61,7 +63,7 @@ dmstor(const char *is, char **rs) {
 			continue;
 		}
 		if (n < nl) {
-			pj_errno = -16;
+			pj_ctx_set_errno( ctx, -16 );
 			return HUGE_VAL;
 		}
 		v += tv * vm[n];
