@@ -1,6 +1,3 @@
-#ifndef lint
-static const char SCCSID[]="@(#)geod_set.c	4.8	95/09/23	GIE	REL";
-#endif
 
 #define _IN_GEOD_SET
 
@@ -24,18 +21,18 @@ geod_set(int argc, char **argv) {
 		else
 			start = curr = pj_mkparam(argv[i]);
 	/* set elliptical parameters */
-	if (pj_ell_set(start, &geod_a, &es)) emess(1,"ellipse setup failure");
+	if (pj_ell_set(pj_get_default_ctx(),start, &geod_a, &es)) emess(1,"ellipse setup failure");
 	/* set units */
-	if (name = pj_param(start, "sunits").s) {
+	if ((name = pj_param(NULL,start, "sunits").s) != NULL) {
 		char *s;
-
-		for (i = 0; (s = pj_units[i].id) && strcmp(name, s) ; ++i) ;
+                struct PJ_UNITS *unit_list = pj_get_units_ref();
+		for (i = 0; (s = unit_list[i].id) && strcmp(name, s) ; ++i) ;
 		if (!s)
 			emess(1,"%s unknown unit conversion id", name);
-		fr_meter = 1. / (to_meter = atof(pj_units[i].to_meter));
+		fr_meter = 1. / (to_meter = atof(unit_list[i].to_meter));
 	} else
 		to_meter = fr_meter = 1.;
-	if (ellipse = es != 0.) {
+	if ((ellipse = es) != 0.) {
 		onef = sqrt(1. - es);
 		geod_f = 1 - onef;
 		f2 = geod_f/2;
@@ -46,27 +43,27 @@ geod_set(int argc, char **argv) {
 		geod_f = f2 = f4 = f64 = 0.;
 	}
 	/* check if line or arc mode */
-	if (pj_param(start, "tlat_1").i) {
+	if (pj_param(NULL,start, "tlat_1").i) {
 		double del_S;
 #undef f
-		phi1 = pj_param(start, "rlat_1").f;
-		lam1 = pj_param(start, "rlon_1").f;
-		if (pj_param(start, "tlat_2").i) {
-			phi2 = pj_param(start, "rlat_2").f;
-			lam2 = pj_param(start, "rlon_2").f;
+		phi1 = pj_param(NULL,start, "rlat_1").f;
+		lam1 = pj_param(NULL,start, "rlon_1").f;
+		if (pj_param(NULL,start, "tlat_2").i) {
+			phi2 = pj_param(NULL,start, "rlat_2").f;
+			lam2 = pj_param(NULL,start, "rlon_2").f;
 			geod_inv();
 			geod_pre();
-		} else if (geod_S = pj_param(start, "dS").f) {
-			al12 = pj_param(start, "rA").f;
+		} else if ((geod_S = pj_param(NULL,start, "dS").f) != 0.) {
+			al12 = pj_param(NULL,start, "rA").f;
 			geod_pre();
 			geod_for();
 		} else emess(1,"incomplete geodesic/arc info");
-		if ((n_alpha = pj_param(start, "in_A").i) > 0) {
-			if (!(del_alpha = pj_param(start, "rdel_A").f))
+		if ((n_alpha = pj_param(NULL,start, "in_A").i) > 0) {
+			if (!(del_alpha = pj_param(NULL,start, "rdel_A").f))
 				emess(1,"del azimuth == 0");
-		} else if (del_S = fabs(pj_param(start, "ddel_S").f)) {
+		} else if ((del_S = fabs(pj_param(NULL,start, "ddel_S").f)) != 0.) {
 			n_S = geod_S / del_S + .5;
-		} else if ((n_S = pj_param(start, "in_S").i) <= 0)
+		} else if ((n_S = pj_param(NULL,start, "in_S").i) <= 0)
 			emess(1,"no interval divisor selected");
 	}
 	/* free up linked list */
