@@ -1,7 +1,4 @@
 /* determine latitude angle phi-2 */
-#ifndef lint
-static const char SCCSID[]="@(#)pj_phi2.c	4.3	93/06/12	GIE	REL";
-#endif
 #include <projects.h>
 
 #define HALFPI		1.5707963267948966
@@ -9,7 +6,7 @@ static const char SCCSID[]="@(#)pj_phi2.c	4.3	93/06/12	GIE	REL";
 #define N_ITER 15
 
 	double
-pj_phi2(double ts, double e) {
+pj_phi2(projCtx ctx, double ts, double e) {
 	double eccnth, Phi, con, dphi;
 	int i;
 
@@ -23,6 +20,6 @@ pj_phi2(double ts, double e) {
 		Phi += dphi;
 	} while ( fabs(dphi) > TOL && --i);
 	if (i <= 0)
-		pj_errno = -18;
+		pj_ctx_set_errno( ctx, -18 );
 	return Phi;
 }
