@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: pj_transform.c,v 1.24 2007/12/03 15:48:20 fwarmerdam Exp $
+ * $Id: pj_transform.c 2000 2011-05-10 17:06:33Z warmerdam $
  *
  * Project:  PROJ.4
  * Purpose:  Perform overall coordinate system to coordinate system 
@@ -27,108 +27,18 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: pj_transform.c,v $
- * Revision 1.24  2007/12/03 15:48:20  fwarmerdam
- * Improve WGS84 ES precision to avoid unnecesary transformation (#1531)
- *
- * Revision 1.23  2007/11/26 00:21:59  fwarmerdam
- * Modified PJ structure to hold a_orig, es_orig, ellipsoid definition before
- * adjustment for spherical projections.
- * Modified pj_datum_transform() to use the original ellipsoid parameters,
- * not the ones adjusted for spherical projections.
- * Modified pj_datum_transform() to not attempt any datum shift via
- * geocentric coordinates if the source *or* destination are raw ellipsoids
- * (ie. PJD_UNKNOWN).  All per PROJ bug #1602, GDAL bug #2025.
- *
- * Revision 1.22  2007/09/11 20:32:25  fwarmerdam
- * mark the transient error array const
- *
- * Revision 1.21  2007/09/11 20:19:36  fwarmerdam
- * avoid use of static variables to make reentrant
- *
- * Revision 1.20  2006/10/12 21:04:39  fwarmerdam
- * Added experimental +lon_wrap argument to set a "center point" for
- * longitude wrapping of longitude values coming out of pj_transform().
- *
- * Revision 1.19  2006/05/10 19:23:47  fwarmerdam
- * Don't apply to_meter in pj_transform() if the value is HUGE_VAL.
- *
- * Revision 1.18  2006/05/01 21:13:54  fwarmerdam
- * make out of range errors in geodetic to geocentric a transient error
- *
- * Revision 1.17  2006/03/20 17:54:34  fwarmerdam
- * pj_geodetic_to_geocentric returns -14 now for lat out of range
- *
- * Revision 1.16  2006/02/17 02:26:14  fwarmerdam
- * ERANGE/EDOM treated as transient errors
- *
- * Revision 1.15  2005/12/04 14:47:37  fwarmerdam
- * use symbolic names as per patch from Martin Vermeer
- *
- * Revision 1.14  2004/11/05 06:05:11  fwarmerdam
- * Fixed pj_geocentric_to_geodetic() to not try and process HUGE_VAL values
- * (those that have failed some previous transform step).  Related to bug:5B
- *     http://bugzilla.remotesensing.org/show_bug.cgi?id=642
- *
- * Revision 1.13  2004/10/25 15:34:36  fwarmerdam
- * make names of geodetic funcs from geotrans unique
- *
- * Revision 1.12  2004/05/03 19:45:23  warmerda
- * Altered so that raw ellpses are treated as a essentially having a
- * +towgs84=0,0,0 specification so ellpisoid shifts are applied.
- * Fixed so that prime meridian shifts are applied if the coordinate system is
- * not lat/long (ie. if it is projected).  This fixes:
- * http://bugzilla.remotesensing.org/show_bug.cgi?id=510
- *
- * Revision 1.11  2004/01/24 09:37:19  warmerda
- * pj_transform() will no longer return an error code if any of the points are
- * transformable.  In this case the application is expected to check for
- * HUGE_VAL to identify failed points.
- * As part of the implementation, I added a list of pj_errno values that
- * are transient (ie per-point) rather than indicating global failure for the
- * coordinate system definition.  We use this in deciding which pj_fwd and
- * pj_inv error codes are really fatal and should be reported.
- *
- * Revision 1.10  2003/08/21 02:09:06  warmerda
- * added a bunch of HUGE_VAL checking
- *
- * Revision 1.9  2003/03/26 16:52:30  warmerda
- * added check that an inverse transformation func exists
- *
- * Revision 1.8  2002/12/14 20:35:43  warmerda
- * implement units support for geocentric coordinates
- *
- * Revision 1.7  2002/12/14 20:14:35  warmerda
- * added geocentric support
- *
- * Revision 1.6  2002/12/09 16:01:02  warmerda
- * added prime meridian support
- *
- * Revision 1.5  2002/12/01 19:25:26  warmerda
- * applied fix for 7 param shift in pj_geocentric_from_wgs84, see bug 194
- *
- * Revision 1.4  2002/02/15 14:30:36  warmerda
- * provide default Z array if none passed in in pj_datum_transform()
- *
- * Revision 1.3  2001/04/04 21:13:21  warmerda
- * do arcsecond/radian and ppm datum parm transformation in pj_set_datum()
- *
- * Revision 1.2  2001/04/04 16:08:08  warmerda
- * rewrote 7 param datum shift to match EPSG:9606, now works with example
- *
- * Revision 1.1  2000/07/06 23:32:27  warmerda
- * New
- *
- */
+ *****************************************************************************/
 
 #include <projects.h>
 #include <string.h>
 #include <math.h>
 #include "geocent.h"
 
-PJ_CVSID("$Id: pj_transform.c,v 1.24 2007/12/03 15:48:20 fwarmerdam Exp $");
+PJ_CVSID("$Id: pj_transform.c 2000 2011-05-10 17:06:33Z warmerdam $");
+
+static int pj_adjust_axis( projCtx ctx, const char *axis, int denormalize_flag,
+                           long point_count, int point_offset, 
+                           double *x, double *y, double *z );
 
 #ifndef SRS_WGS84_SEMIMAJOR
 #define SRS_WGS84_SEMIMAJOR 6378137.0
@@ -157,13 +67,13 @@ PJ_CVSID("$Id: pj_transform.c,v 1.24 2007/12/03 15:48:20 fwarmerdam Exp $");
 ** list or something, but while experimenting with it this should be fine. 
 */
 
-static const int transient_error[45] = {
+static const int transient_error[50] = {
     /*             0  1  2  3  4  5  6  7  8  9   */
     /* 0 to 9 */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
     /* 10 to 19 */ 0, 0, 0, 0, 1, 1, 0, 1, 1, 1,  
     /* 20 to 29 */ 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 
-    /* 30 to 39 */ 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 
-    /* 40 to 44 */ 0, 0, 0, 0, 0 };
+    /* 30 to 39 */ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+    /* 40 to 49 */ 0, 0, 0, 0, 0, 0, 0, 0, 1, 0 };
 
 /************************************************************************/
 /*                            pj_transform()                            */
@@ -179,13 +89,37 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
 
 {
     long      i;
-    int       need_datum_shift;
+    int       err;
 
-    pj_errno = 0;
+    srcdefn->ctx->last_errno = 0;
+    dstdefn->ctx->last_errno = 0;
 
     if( point_offset == 0 )
         point_offset = 1;
 
+/* -------------------------------------------------------------------- */
+/*      Transform unusual input coordinate axis orientation to          */
+/*      standard form if needed.                                        */
+/* -------------------------------------------------------------------- */
+    if( strcmp(srcdefn->axis,"enu") != 0 )
+    {
+        int err;
+
+        err = pj_adjust_axis( srcdefn->ctx, srcdefn->axis, 
+                              0, point_count, point_offset, x, y, z );
+        if( err != 0 )
+            return err;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Transform Z to meters if it isn't already.                      */
+/* -------------------------------------------------------------------- */
+    if( srcdefn->vto_meter != 1.0 && z != NULL )
+    {
+        for( i = 0; i < point_count; i++ )
+            z[point_offset*i] *= srcdefn->vto_meter;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Transform geocentric source coordinates to lat/long.            */
 /* -------------------------------------------------------------------- */
@@ -193,7 +127,7 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
     {
         if( z == NULL )
         {
-            pj_errno = PJD_ERR_GEOCENTRIC;
+            pj_ctx_set_errno( pj_get_ctx(srcdefn), PJD_ERR_GEOCENTRIC);
             return PJD_ERR_GEOCENTRIC;
         }
 
@@ -209,10 +143,11 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
             }
         }
 
-        if( pj_geocentric_to_geodetic( srcdefn->a_orig, srcdefn->es_orig,
-                                       point_count, point_offset, 
-                                       x, y, z ) != 0) 
-            return pj_errno;
+        err = pj_geocentric_to_geodetic( srcdefn->a_orig, srcdefn->es_orig,
+                                         point_count, point_offset, 
+                                         x, y, z );
+        if( err != 0 )
+            return err;
     }
 
 /* -------------------------------------------------------------------- */
@@ -223,13 +158,10 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
     {
         if( srcdefn->inv == NULL )
         {
-            pj_errno = -17; /* this isn't correct, we need a no inverse err */
-            if( getenv( "PROJ_DEBUG" ) != NULL )
-            {
-                fprintf( stderr, 
-                       "pj_transform(): source projection not invertable\n" );
-            }
-            return pj_errno;
+            pj_ctx_set_errno( pj_get_ctx(srcdefn), -17 );
+            pj_log( pj_get_ctx(srcdefn), PJ_LOG_ERROR, 
+                    "pj_transform(): source projection not invertable" );
+            return -17;
         }
 
         for( i = 0; i < point_count; i++ )
@@ -244,12 +176,14 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
                 continue;
 
             geodetic_loc = pj_inv( projected_loc, srcdefn );
-            if( pj_errno != 0 )
+            if( srcdefn->ctx->last_errno != 0 )
             {
-                if( (pj_errno != 33 /*EDOM*/ && pj_errno != 34 /*ERANGE*/ )
-                    && (pj_errno > 0 || pj_errno < -44 || point_count == 1
-                        || transient_error[-pj_errno] == 0 ) )
-                    return pj_errno;
+                if( (srcdefn->ctx->last_errno != 33 /*EDOM*/ 
+                     && srcdefn->ctx->last_errno != 34 /*ERANGE*/ )
+                    && (srcdefn->ctx->last_errno > 0 
+                        || srcdefn->ctx->last_errno < -44 || point_count == 1
+                        || transient_error[-srcdefn->ctx->last_errno] == 0 ) )
+                    return srcdefn->ctx->last_errno;
                 else
                 {
                     geodetic_loc.u = HUGE_VAL;
@@ -274,13 +208,44 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
         }
     }
 
+/* -------------------------------------------------------------------- */
+/*      Do we need to translate from geoid to ellipsoidal vertical      */
+/*      datum?                                                          */
+/* -------------------------------------------------------------------- */
+    if( srcdefn->has_geoid_vgrids )
+    {
+        if( pj_apply_vgridshift( srcdefn, "sgeoidgrids", 
+                                 &(srcdefn->vgridlist_geoid), 
+                                 &(srcdefn->vgridlist_geoid_count),
+                                 0, point_count, point_offset, x, y, z ) != 0 )
+            return pj_ctx_get_errno(srcdefn->ctx);
+    }
+        
 /* -------------------------------------------------------------------- */
 /*      Convert datums if needed, and possible.                         */
 /* -------------------------------------------------------------------- */
     if( pj_datum_transform( srcdefn, dstdefn, point_count, point_offset, 
                             x, y, z ) != 0 )
-        return pj_errno;
+    {
+        if( srcdefn->ctx->last_errno != 0 )
+            return srcdefn->ctx->last_errno;
+        else
+            return dstdefn->ctx->last_errno;
+    }
 
+/* -------------------------------------------------------------------- */
+/*      Do we need to translate from geoid to ellipsoidal vertical      */
+/*      datum?                                                          */
+/* -------------------------------------------------------------------- */
+    if( dstdefn->has_geoid_vgrids )
+    {
+        if( pj_apply_vgridshift( dstdefn, "sgeoidgrids", 
+                                 &(dstdefn->vgridlist_geoid), 
+                                 &(dstdefn->vgridlist_geoid_count),
+                                 1, point_count, point_offset, x, y, z ) != 0 )
+            return dstdefn->ctx->last_errno;
+    }
+        
 /* -------------------------------------------------------------------- */
 /*      But if they are staying lat long, adjust for the prime          */
 /*      meridian if there is one in effect.                             */
@@ -302,7 +267,7 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
     {
         if( z == NULL )
         {
-            pj_errno = PJD_ERR_GEOCENTRIC;
+            pj_ctx_set_errno( dstdefn->ctx, PJD_ERR_GEOCENTRIC );
             return PJD_ERR_GEOCENTRIC;
         }
 
@@ -340,12 +305,14 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
                 continue;
 
             projected_loc = pj_fwd( geodetic_loc, dstdefn );
-            if( pj_errno != 0 )
+            if( dstdefn->ctx->last_errno != 0 )
             {
-                if( (pj_errno != 33 /*EDOM*/ && pj_errno != 34 /*ERANGE*/ )
-                    && (pj_errno > 0 || pj_errno < -44 || point_count == 1
-                        || transient_error[-pj_errno] == 0 ) )
-                    return pj_errno;
+                if( (dstdefn->ctx->last_errno != 33 /*EDOM*/ 
+                     && dstdefn->ctx->last_errno != 34 /*ERANGE*/ )
+                    && (dstdefn->ctx->last_errno > 0 
+                        || dstdefn->ctx->last_errno < -44 || point_count == 1
+                        || transient_error[-dstdefn->ctx->last_errno] == 0 ) )
+                    return dstdefn->ctx->last_errno;
                 else
                 {
                     projected_loc.u = HUGE_VAL;
@@ -362,20 +329,43 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
 /*      If a wrapping center other than 0 is provided, rewrap around    */
 /*      the suggested center (for latlong coordinate systems only).     */
 /* -------------------------------------------------------------------- */
-    else if( dstdefn->is_latlong && dstdefn->long_wrap_center != 0 )
+    else if( dstdefn->is_latlong && dstdefn->is_long_wrap_set )
     {
         for( i = 0; i < point_count; i++ )
         {
             if( x[point_offset*i] == HUGE_VAL )
                 continue;
 
-            while( x[point_offset*i] < dstdefn->long_wrap_center - HALFPI )
-                x[point_offset*i] += PI;
-            while( x[point_offset*i] > dstdefn->long_wrap_center + HALFPI )
-                x[point_offset*i] -= PI;
+            while( x[point_offset*i] < dstdefn->long_wrap_center - PI )
+                x[point_offset*i] += TWOPI;
+            while( x[point_offset*i] > dstdefn->long_wrap_center + PI )
+                x[point_offset*i] -= TWOPI;
         }
     }
 
+/* -------------------------------------------------------------------- */
+/*      Transform Z from meters if needed.                              */
+/* -------------------------------------------------------------------- */
+    if( dstdefn->vto_meter != 1.0 && z != NULL )
+    {
+        for( i = 0; i < point_count; i++ )
+            z[point_offset*i] *= dstdefn->vfr_meter;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Transform normalized axes into unusual output coordinate axis   */
+/*      orientation if needed.                                          */
+/* -------------------------------------------------------------------- */
+    if( strcmp(dstdefn->axis,"enu") != 0 )
+    {
+        int err;
+
+        err = pj_adjust_axis( dstdefn->ctx, dstdefn->axis, 
+                              1, point_count, point_offset, x, y, z );
+        if( err != 0 )
+            return err;
+    }
+
     return 0;
 }
 
@@ -391,8 +381,7 @@ int pj_geodetic_to_geocentric( double a, double es,
     double b;
     int    i;
     GeocentricInfo gi;
-
-    pj_errno = 0;
+    int ret_errno = 0;
 
     if( es == 0.0 )
         b = a;
@@ -401,8 +390,7 @@ int pj_geodetic_to_geocentric( double a, double es,
 
     if( pj_Set_Geocentric_Parameters( &gi, a, b ) != 0 )
     {
-        pj_errno = PJD_ERR_GEOCENTRIC;
-        return pj_errno;
+        return PJD_ERR_GEOCENTRIC;
     }
 
     for( i = 0; i < point_count; i++ )
@@ -415,13 +403,13 @@ int pj_geodetic_to_geocentric( double a, double es,
         if( pj_Convert_Geodetic_To_Geocentric( &gi, y[io], x[io], z[io], 
                                                x+io, y+io, z+io ) != 0 )
         {
-            pj_errno = -14;
+            ret_errno = -14;
             x[io] = y[io] = HUGE_VAL;
             /* but keep processing points! */
         }
     }
 
-    return pj_errno;
+    return ret_errno;
 }
 
 /************************************************************************/
@@ -444,8 +432,7 @@ int pj_geocentric_to_geodetic( double a, double es,
 
     if( pj_Set_Geocentric_Parameters( &gi, a, b ) != 0 )
     {
-        pj_errno = PJD_ERR_GEOCENTRIC;
-        return pj_errno;
+        return PJD_ERR_GEOCENTRIC;
     }
 
     for( i = 0; i < point_count; i++ )
@@ -501,8 +488,8 @@ int pj_compare_datums( PJ *srcdefn, PJ *dstdefn )
     }
     else if( srcdefn->datum_type == PJD_GRIDSHIFT )
     {
-        return strcmp( pj_param(srcdefn->params,"snadgrids").s,
-                       pj_param(dstdefn->params,"snadgrids").s ) == 0;
+        return strcmp( pj_param(srcdefn->ctx, srcdefn->params,"snadgrids").s,
+                       pj_param(dstdefn->ctx, dstdefn->params,"snadgrids").s ) == 0;
     }
     else
         return 1;
@@ -519,8 +506,6 @@ int pj_geocentric_to_wgs84( PJ *defn,
 {
     int       i;
 
-    pj_errno = 0;
-
     if( defn->datum_type == PJD_3PARAM )
     {
         for( i = 0; i < point_count; i++ )
@@ -569,8 +554,6 @@ int pj_geocentric_from_wgs84( PJ *defn,
 {
     int       i;
 
-    pj_errno = 0;
-
     if( defn->datum_type == PJD_3PARAM )
     {
         for( i = 0; i < point_count; i++ )
@@ -624,8 +607,6 @@ int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
     double      src_a, src_es, dst_a, dst_es;
     int         z_is_temp = FALSE;
 
-    pj_errno = 0;
-
 /* -------------------------------------------------------------------- */
 /*      We cannot do any meaningful datum transformation if either      */
 /*      the source or destination are of an unknown datum type          */
@@ -659,7 +640,7 @@ int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
         z_is_temp = TRUE;
     }
 
-#define CHECK_RETURN {if( pj_errno != 0 && (pj_errno > 0 || transient_error[-pj_errno] == 0) ) { if( z_is_temp ) pj_dalloc(z); return pj_errno; }}
+#define CHECK_RETURN(defn) {if( defn->ctx->last_errno != 0 && (defn->ctx->last_errno > 0 || transient_error[-defn->ctx->last_errno] == 0) ) { if( z_is_temp ) pj_dalloc(z); return defn->ctx->last_errno; }}
 
 /* -------------------------------------------------------------------- */
 /*	If this datum requires grid shifts, then apply it to geodetic   */
@@ -667,9 +648,8 @@ int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
 /* -------------------------------------------------------------------- */
     if( srcdefn->datum_type == PJD_GRIDSHIFT )
     {
-        pj_apply_gridshift( pj_param(srcdefn->params,"snadgrids").s, 0, 
-                            point_count, point_offset, x, y, z );
-        CHECK_RETURN;
+        pj_apply_gridshift_2( srcdefn, 0, point_count, point_offset, x, y, z );
+        CHECK_RETURN(srcdefn);
 
         src_a = SRS_WGS84_SEMIMAJOR;
         src_es = SRS_WGS84_ESQUARED;
@@ -680,7 +660,7 @@ int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
         dst_a = SRS_WGS84_SEMIMAJOR;
         dst_es = SRS_WGS84_ESQUARED;
     }
-        
+
 /* ==================================================================== */
 /*      Do we need to go through geocentric coordinates?                */
 /* ==================================================================== */
@@ -693,9 +673,10 @@ int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
 /* -------------------------------------------------------------------- */
 /*      Convert to geocentric coordinates.                              */
 /* -------------------------------------------------------------------- */
-        pj_geodetic_to_geocentric( src_a, src_es,
-                                   point_count, point_offset, x, y, z );
-        CHECK_RETURN;
+        srcdefn->ctx->last_errno = 
+            pj_geodetic_to_geocentric( src_a, src_es,
+                                       point_count, point_offset, x, y, z );
+        CHECK_RETURN(srcdefn);
 
 /* -------------------------------------------------------------------- */
 /*      Convert between datums.                                         */
@@ -704,22 +685,23 @@ int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
             || srcdefn->datum_type == PJD_7PARAM )
         {
             pj_geocentric_to_wgs84( srcdefn, point_count, point_offset,x,y,z);
-            CHECK_RETURN;
+            CHECK_RETURN(srcdefn);
         }
 
         if( dstdefn->datum_type == PJD_3PARAM 
             || dstdefn->datum_type == PJD_7PARAM )
         {
             pj_geocentric_from_wgs84( dstdefn, point_count,point_offset,x,y,z);
-            CHECK_RETURN;
+            CHECK_RETURN(dstdefn);
         }
 
 /* -------------------------------------------------------------------- */
 /*      Convert back to geodetic coordinates.                           */
 /* -------------------------------------------------------------------- */
-        pj_geocentric_to_geodetic( dst_a, dst_es,
-                                   point_count, point_offset, x, y, z );
-        CHECK_RETURN;
+        dstdefn->ctx->last_errno = 
+            pj_geocentric_to_geodetic( dst_a, dst_es,
+                                       point_count, point_offset, x, y, z );
+        CHECK_RETURN(dstdefn);
     }
 
 /* -------------------------------------------------------------------- */
@@ -727,9 +709,8 @@ int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
 /* -------------------------------------------------------------------- */
     if( dstdefn->datum_type == PJD_GRIDSHIFT )
     {
-        pj_apply_gridshift( pj_param(dstdefn->params,"snadgrids").s, 1,
-                            point_count, point_offset, x, y, z );
-        CHECK_RETURN;
+        pj_apply_gridshift_2( dstdefn, 1, point_count, point_offset, x, y, z );
+        CHECK_RETURN(dstdefn);
     }
 
     if( z_is_temp )
@@ -738,3 +719,108 @@ int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
     return 0;
 }
 
+/************************************************************************/
+/*                           pj_adjust_axis()                           */
+/*                                                                      */
+/*      Normalize or de-normalized the x/y/z axes.  The normal form     */
+/*      is "enu" (easting, northing, up).                               */
+/************************************************************************/
+static int pj_adjust_axis( projCtx ctx, 
+                           const char *axis, int denormalize_flag,
+                           long point_count, int point_offset, 
+                           double *x, double *y, double *z )
+
+{
+    double x_in, y_in, z_in = 0.0;
+    int i, i_axis;
+
+    if( !denormalize_flag )
+    {
+        for( i = 0; i < point_count; i++ )
+        {
+            x_in = x[point_offset*i];
+            y_in = y[point_offset*i];
+            if( z )
+                z_in = z[point_offset*i];
+     
+            for( i_axis = 0; i_axis < 3; i_axis++ )
+            {
+                double value;
+
+                if( i_axis == 0 )
+                    value = x_in;
+                else if( i_axis == 1 )
+                    value = y_in;
+                else
+                    value = z_in;
+                
+                switch( axis[i_axis] )
+                {
+                  case 'e':
+                    x[point_offset*i] = value; break;
+                  case 'w':
+                    x[point_offset*i] = -value; break;
+                  case 'n':
+                    y[point_offset*i] = value; break;
+                  case 's':
+                    y[point_offset*i] = -value; break;
+                  case 'u':
+                    if( z ) z[point_offset*i] = value; break;
+                  case 'd':
+                    if( z ) z[point_offset*i] = -value; break;
+                  default:
+                    pj_ctx_set_errno( ctx, PJD_ERR_AXIS );
+                    return PJD_ERR_AXIS;
+                }
+            } /* i_axis */
+        } /* i (point) */
+    }
+
+    else /* denormalize */
+    {
+        for( i = 0; i < point_count; i++ )
+        {
+            x_in = x[point_offset*i];
+            y_in = y[point_offset*i];
+            if( z )
+                z_in = z[point_offset*i];
+     
+            for( i_axis = 0; i_axis < 3; i_axis++ )
+            {
+                double *target;
+
+                if( i_axis == 2 && z == NULL )
+                    continue;
+
+                if( i_axis == 0 )
+                    target = x;
+                else if( i_axis == 1 )
+                    target = y;
+                else
+                    target = z;
+                
+                switch( axis[i_axis] )
+                {
+                  case 'e':
+                    target[point_offset*i] = x_in; break;
+                  case 'w':
+                    target[point_offset*i] = -x_in; break;
+                  case 'n':
+                    target[point_offset*i] = y_in; break;
+                  case 's':
+                    target[point_offset*i] = -y_in; break;
+                  case 'u':
+                    target[point_offset*i] = z_in; break;
+                  case 'd':
+                    target[point_offset*i] = -z_in; break;
+                  default:
+                    pj_ctx_set_errno( ctx, PJD_ERR_AXIS );
+                    return PJD_ERR_AXIS;
+                }
+            } /* i_axis */
+        } /* i (point) */
+    }
+    
+    return 0;
+}
+
