@@ -4,7 +4,7 @@
 ** Copyright (c) 2003, 2006   Gerald I. Evenden
 */
 static const char
-LIBPROJ_ID[] = "$Id: proj_mdist.c,v 1.1 2006/10/18 05:21:31 fwarmerdam Exp $";
+LIBPROJ_ID[] = "$Id: proj_mdist.c 1856 2010-06-11 03:26:04Z warmerdam $";
 /*
 ** Permission is hereby granted, free of charge, to any person obtaining
 ** a copy of this software and associated documentation files (the
@@ -102,7 +102,7 @@ proj_mdist(double phi, double sphi, double cphi, const void *b) {
 	return(D + sc * sum);
 }
 	double
-proj_inv_mdist(double dist, const void *b) {
+proj_inv_mdist(projCtx ctx, double dist, const void *b) {
 	double s, t, phi, k;
 	int i;
 
@@ -118,15 +118,6 @@ proj_inv_mdist(double dist, const void *b) {
 			return phi;
 	}
 		/* convergence failed */
-	pj_errno = -17;
+	pj_ctx_set_errno(ctx, -17);
 	return phi;
 }
-/* Revision Log:
-** $Log: proj_mdist.c,v $
-** Revision 1.1  2006/10/18 05:21:31  fwarmerdam
-** New
-**
-** Revision 3.1  2006/01/11 01:38:18  gie
-** Initial
-**
-*/
